/* Copyright 2010 AlarmPoint Systems */
importPackage(java.lang);
importPackage(java.io);
importPackage(Packages.com.alarmpoint.spectrum.cli);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);
importClass(Packages.org.apache.log4j.Logger);

var xStream = new XStream(new PureJavaReflectionProvider())

// SPECTRUM CLI commands used in this integration
var ack_cmd = "AlarmAck";
var clear_cmd = "AlarmClear";
var update_cmd = "updatealarm";

//SPECTRUM Attribute IDs
var AlarmStatus = "0x11f4f";

// Location of SPECTRUM AlarmNotifier
var cmdPath;
var showPath;

// Array of commands to run as part of an AlarmPoint Response
var commands;
var types;

var incidentId;
var loggingHeader = "caspectrum.js: ";
var annotationPrefix = "[AlarmPoint] ";

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by AlarmPoint.  The messages may be responses to
 * previous submissions, or they may be requests to the Integration Service
 * that originate from AlarmPoint
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * either via the Integration Service's outbound queues or directly as a
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml)
{ 
  var method = apxml.getMethod();
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  }
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  }
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}

/**
 * OK responses are returned by AlarmPoint for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * This implementation just logs the ack.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  logInfo("Submission of transaction [" + id + "] succeeded.");
}

/**
 * ERROR responses are returned by AlarmPoint for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * This implementation just logs the error.
 */
function handleERROR(apxml)
{
  var transaction = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE);
  logError("Submission of transaction " + transaction + "failed with error code " + code);
}

/**
 * The main routine for handling Send Requests (ie externalServiceMessage messages) from AlarmPoint
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{
  logDebug( "handleSend received apxml: " + apxml ); 
  
  // handle annotations coming from callout APS script 
  // they are formatted differently 
  // the presence of the message token means this message 
  // is a callout annotation. 
  var message = apxml.getValue("message");
  if ( message != null )
  {
    incidentId = apxml.getValue("incident_id");
    return handleCalloutAnnotate( incidentId, message );
  }
  else
  {
    handleResponse(apxml);
  }
}

/**
 * handles annotations sent from the Callout APS script
 */
function handleCalloutAnnotate( incidentId, message )
{ 
  logNotification("[Alarm " + incidentId + "]" + message);
}

/**
 * The main routine for handling Requests (ie externalServiceRequest2 messages) from AlarmPoint
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleRequest(apxml)
{
  logDebug( "handleRequest received apxml: " + apxml ); 

  var responseApXml = initializeResponse(apxml);

  try
  {
    var response = handleResponse(apxml);
    responseApXml.setToken("apiaResult", response, APXMLToken.Type.STRING);
  }
  catch (e)
  {
    logError("Caught exception processing request. Exception: " + e)
    responseApXml.setToken("apiaResultException", e.toString(), APXMLToken.Type.STRING);
  }
  return responseApXml;
}

/**
 * Run appropriate SPECTRUM CLI command based on user response from AlarmPoint notification
 */  
function handleResponse(apxml)
{
  initvariables();
  return buildCliCommands(apxml);
}

/**
 * This method sets the variables representing the CLI commands to be run based
 * on operating system.
 */
function initvariables()
{
  var os = System.getProperty("os.name");
  if (os.toLowerCase().startsWith("win"))
  {
    cmdPath = "C:\\win32app\\SPECTRUM\\Notifier";
    showPath = "C:\\win32app\\SPECTRUM\\vnmsh";
    logInfo("Setting Windows type path");
  }
  else
  {
    cmdPath = "/usr/local/spectrum";
    showPath = "/usr/local/spectrum/vnmsh";
    logInfo("Setting Unix type path");
  }
}

/**
 * This method builds string representations of the SPECTRUM CLI commands to be run based on
 * the response of an AlarmPoint user.
 * 
 * If the message from AlarmPoint is not understood < ... >  
 */
function buildCliCommands(apxml)
{
  // Parse APXML message to determine response
  incidentId = apxml.getValue("incident_id");
  var requestText = apxml.getValue("request_text");
  var messageText = apxml.getValue("message_text");
  var landscape;
  var mhandle;

  if (!incidentId)
  {
     var errorMessage = "Request Script Failed: Request not associated with an Incident";
     logError(errorMessage);
     return errorMessage;  
  }
  
  logDebug("incident_id: " + incidentId);
  logDebug("requestText: " + requestText);
  logDebug("messageText: " + messageText);

  if  (requestText != "annotate")
  {
    // deserialize the additional tokens
    if (apxml.getValue("additionalTokens") != null && apxml.getValue("additionalTokens") != "") 
    {
      var additionalTokens = xStream.fromXML(apxml.getValue("additionalTokens")); // LinkedHashMap
      
      logDebug( "additionalTokens: " + additionalTokens);
      landscape = additionalTokens.get("landscape"); 
      mhandle = additionalTokens.get("mhandle"); 
    }
    else
    {
      var errorMessage = "additionalTokens is missing from apxml";
      logDebug(errorMessage);
      return errorMessage; 
    }
  }

  logInfo("Processing " + requestText + " type response");
  // decide what program we need to run
  // nothing runs if we don't recognise the request text
  var responseType = "java.lang.String";
  var response = null;
  
  var ack_params = " -a " + incidentId + " -l " + landscape;
  var clear_params = " -a " + incidentId + " -l " + landscape;
  var update_params = " " + mhandle + " " + incidentId + " " + AlarmStatus + " \"" + annotationPrefix + messageText + "\"";
  var notificationMessage = "[Alarm " + incidentId + "]" + messageText;

  if (requestText == "acknowledge")
  {
      response = executeSpectrumCli(cmdPath, ack_cmd + ack_params, responseType);
      executeSpectrumCli(cmdPath, update_cmd + update_params, null);
      logNotification(notificationMessage);
      return response;
  }
  else if (requestText == "clear")
  {
      executeSpectrumCli(cmdPath, update_cmd + update_params, null);
      response = executeSpectrumCli(cmdPath, clear_cmd + clear_params, responseType);
      logNotification(notificationMessage);
      return response;
  }
  // Handles both ignore and annotate responses
  else if (requestText == "annotate")
  {   
      logNotification(notificationMessage);
  }
  else
  {
      var errorMessage = "Unexpected request_text value: " + requestText; 
      logError(errorMessage);
      return errorMessage;
  }
}

/**
 * Run the Spectrum CLI command and wait for its response.
 * Read the response and log it. 
 **/ 
function executeSpectrumCli(path, command, type)
{
  var response = null;
  try 
  {
     logInfo("About the run cli commands");
     // Execute the program using path as current directory
     var cliMgr = new SpectrumCLIManager(path);
     logDebug("cli - " + command);
     response = cliMgr.querySpectrum(command, type);
     logDebug("response - " + response);
  }
  catch(e)
  {
     logError("Attempt to update Alarm ID " + incidentId + " failed: " + e);
  }
  return response;
}

/**
 * Called from the web application to run a show command
 */
function show(params)
{
  initvariables();
  executeSpectrumCli(showPath, "connect", null);
  var response = executeSpectrumCli(showPath, params.remove("cli_action"), params.remove("cli_type"));
  executeSpectrumCli(showPath, "disconnect", null);

  return response;
}

/**
 * Initializes the response to be returned to the device
 * @param apxml APXML containing the request information
 * @return initialized instance of the response APXML
 */
function initializeResponse(apxml)
{
  var response = ServiceAPI.createAPXML();
  response.setMethod(APXMLMessage.RESPONSE_METHOD);
  response.setSubclass(APXMLMessage.RESPONSE_SUBCLASS);
  
  response.setToken(APXMLMessage.REQUEST_ID, apxml.getValue(APXMLMessage.REQUEST_ID), APXMLToken.Type.STRING);
  response.setToken(APXMLMessage.DESTINATION, apxml.getValue(APXMLMessage.ORIGINATOR), APXMLToken.Type.STRING);

  if (apxml.hasToken(APXMLMessage.APIA_PRIORITY)) 
  {
    response.setToken(APXMLMessage.APIA_PRIORITY, apxml.getValue(APXMLMessage.APIA_PRIORITY));
  }
  else
  {
    response.setToken(APXMLMessage.APIA_PRIORITY, APXMLMessage.APIA_PRIORITY_HIGH);
  }
  return response;
}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by APClient.  The messages may be requests to perform
 * local activity, or they may be requests to make submissions to AlarmPoint.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  var aid = apxml.getValue("aid");
  var mhandle = apxml.getValue("mhandle");
  var incidentId = apxml.getValue("incident_id");
  var sentMessage = "Alarm " + aid + " sent to AlarmPoint";
  var update_params = " " + mhandle + " " + incidentId + " " + AlarmStatus + " \"" + annotationPrefix + sentMessage + "\"";
  initvariables();
  
  logNotification(sentMessage);
  executeSpectrumCli(cmdPath, update_cmd + update_params, null);
  
  return apxml;
}

/**
 * Logging functions
 */
function logDebug(msg)
{
  ServiceAPI.getLogger().debug(loggingHeader + msg);
}

function logInfo(msg)
{
  ServiceAPI.getLogger().info(loggingHeader + msg);
}

function logError(msg)
{
  ServiceAPI.getLogger().error(loggingHeader + msg);
}

/**
 * This function was created to log all AlarmPoint notification deliveries.
 * The intent is that the log defined by "com.alarmpoint.notification.delivery" would fulfill the role of an audit log.
 */  
function logNotification(msg)
{
  Logger.getLogger("com.alarmpoint.notification.delivery").info(msg);
}
